@@ -21,6 +21,8 @@
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::LaTeX - Convert LaTeX documents and derivates from/to PO files
@@ -307,7 +309,7 @@ register_generic_command("*textcolor,{}{_}");
 register_generic_command("*colorbox,{}{_}");
 register_generic_command("*fcolorbox,{}{}{_}");
 register_generic_command("*pagecolor,{_}");
-register_generic_command("*color,{_}");
+register_generic_command("*color,{}");
 
 # equations/theorems
 register_generic_command("*qedhere,");
@@ -360,6 +362,7 @@ foreach (qw(abstract align align* cases center description displaymath document
 }
 register_generic_environment("tabular,[]{}");
 register_generic_environment("tabular*,{}{}");
+register_generic_environment("tabularx,{}{}");
 register_generic_environment("multicols,{}");
 register_generic_environment("list,{_}{}");
 register_generic_environment("array,[]{}");
@@ -375,7 +378,8 @@ register_generic_environment("thebibliography,{_}");
 # & is the cell separator, \\ is the line separator
 # '\' is escaped twice
 $env_separators{'array'} =
-  $env_separators{'tabular'} = "(?:&|\\\\\\\\)";
+  $env_separators{'tabular'} =
+  $env_separators{'tabularx'} = "(?:&|\\\\\\\\|\\\\hline)";
 
 $env_separators{'trivlist'} =
   $env_separators{'list'} =
