@@ -1,5 +1,4 @@
 # Locale::Po4a::Po -- manipulation of po files
-# $Id: Po.pm,v 1.97 2009-12-30 21:12:31 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -249,7 +248,7 @@ sub read {
     my $linenum=0;
 
     foreach my $msg (split (/\n\n/,$pofile)) {
-        my ($msgid,$msgstr,$comment,$automatic,$reference,$flags,$buffer);
+        my ($msgid,$msgstr,$comment,$previous,$automatic,$reference,$flags,$buffer);
         my ($msgid_plural, $msgstr_plural);
         foreach my $line (split (/\n/,$msg)) {
             $linenum++;
@@ -262,6 +261,9 @@ sub read {
             } elsif ($line =~ /^#, ?(.*)$/) { # flags
                 $flags .= (defined($flags) ? "\n" : "").$1;
 
+            } elsif ($line =~ /^#\| ?(.*)$/) { # previous translation
+                $previous .= (defined($previous) ? "\n" : "").($1||"");
+
             } elsif ($line =~ /^#(.*)$/) {  # Translator comments
                 $comment .= (defined($comment) ? "\n" : "").($1||"");
 
@@ -324,6 +326,7 @@ sub read {
                              'reference' => $reference,
                              'flags'     => $flags,
                              'comment'   => $comment,
+                             'previous'  => $previous,
                              'automatic' => $automatic,
                              'plural'    => 0);
 
@@ -337,6 +340,7 @@ sub read {
                              'reference' => $reference,
                              'flags'     => $flags,
                              'comment'   => $comment,
+                             'previous'  => $previous,
                              'automatic' => $automatic,
                              'plural'    => 1);
         } else {
@@ -350,6 +354,7 @@ sub read {
                              'reference' => $reference,
                              'flags'     => $flags,
                              'comment'   => $comment,
+                             'previous'  => $previous,
                              'automatic' => $automatic);
         }
     }
@@ -392,7 +397,7 @@ sub write{
     print $fh "msgstr ".quote_text($self->{header})."\n\n";
 
 
-    my $buf_msgstr_plural; # USed to keep the first msgstr of plural forms
+    my $buf_msgstr_plural; # Used to keep the first msgstr of plural forms
     my $first=1;
     foreach my $msgid ( sort { ($self->{po}{"$a"}{'pos'}) <=>
                                ($self->{po}{"$b"}{'pos'})
@@ -424,6 +429,9 @@ sub write{
         $output .= "#, ". join(", ", sort split(/\s+/,$self->{po}{$msgid}{'flags'}))."\n"
             if    defined($self->{po}{$msgid}{'flags'})
                && length ($self->{po}{$msgid}{'flags'});
+        $output .= format_comment($self->{po}{$msgid}{'previous'},"| ")
+            if    defined($self->{po}{$msgid}{'previous'})
+               && length ($self->{po}{$msgid}{'previous'});
 
         if (exists $self->{po}{$msgid}{'plural'}) {
             if ($self->{po}{$msgid}{'plural'} == 0) {
@@ -635,6 +643,13 @@ sub gettextize {
         #
         if ($typeorig ne $typetrans){
             $pores->write("gettextization.failed.po");
+            eval {
+               # Recode $trans into current charset, if possible
+               require I18N::Langinfo;
+               I18N::Langinfo->import(qw(langinfo CODESET));
+               my $codeset = langinfo(CODESET());
+               Encode::from_to($trans, $potrans->get_charset, $codeset);
+            };
             die wrap_msg(dgettext("po4a",
                          "po4a gettextization: Structure disparity between ".
                          "original and translated files:\n".
@@ -789,7 +804,7 @@ sub filter {
             # check that we've got a valid field name,
             # and the number it referes to
             # DO NOT CHANGE THE ORDER
-            my @names=qw(msgid msgstr reference flags comment automatic);
+            my @names=qw(msgid msgstr reference flags comment previous automatic);
             my $fieldpos;
             for ($fieldpos = 0;
                  $fieldpos < scalar @names && $field ne $names[$fieldpos];
@@ -849,7 +864,7 @@ sub filter {
                 $pos++;
             }
             # and now, add the code to check this equality
-            $code .= "(\$_[$fieldpos] =~ m/$arg/)";
+            $code .= "(\$_[$fieldpos] =~ m{$arg})";
 
         }
         showmethecode("End of expression")
@@ -877,7 +892,7 @@ sub filter {
          $cpt<$self->count_entries();
          $cpt++) {
 
-        my ($msgid,$ref,$msgstr,$flags,$type,$comment,$automatic);
+        my ($msgid,$ref,$msgstr,$flags,$type,$comment,$previous,$automatic);
 
         $msgid = $self->msgid($cpt);
         $ref=$self->{po}{$msgid}{'reference'};
@@ -886,6 +901,7 @@ sub filter {
         $flags =  $self->{po}{$msgid}{'flags'};
         $type = $self->{po}{$msgid}{'type'};
         $comment = $self->{po}{$msgid}{'comment'};
+        $previous = $self->{po}{$msgid}{'previous'};
         $automatic = $self->{po}{$msgid}{'automatic'};
 
         # DO NOT CHANGE THE ORDER
@@ -895,8 +911,9 @@ sub filter {
                        'type'  => $type,
                        'reference' => $ref,
                        'comment' => $comment,
+                       'previous' => $previous,
                        'automatic' => $automatic)
-               if (apply($msgid,$msgstr,$ref,$flags,$comment,$automatic));
+               if (apply($msgid,$msgstr,$ref,$flags,$comment,$previous,$automatic));
     }
     # delete the apply subroutine
     # otherwise it will be redefined.
@@ -1139,7 +1156,7 @@ sub push {
     my $self=shift;
     my %entry=@_;
 
-    my $validoption="wrap wrapcol type msgid msgstr automatic flags reference";
+    my $validoption="wrap wrapcol type msgid msgstr automatic previous flags reference";
     my %validoption;
 
     map { $validoption{$_}=1 } (split(/ /,$validoption));
@@ -1172,10 +1189,10 @@ sub push {
 sub push_raw {
     my $self=shift;
     my %entry=@_;
-    my ($msgid,$msgstr,$reference,$comment,$automatic,$flags,$type,$transref)=
+    my ($msgid,$msgstr,$reference,$comment,$automatic,$previous,$flags,$type,$transref)=
         ($entry{'msgid'},$entry{'msgstr'},
          $entry{'reference'},$entry{'comment'},$entry{'automatic'},
-         $entry{'flags'},$entry{'type'},$entry{'transref'});
+         $entry{'previous'},$entry{'flags'},$entry{'type'},$entry{'transref'});
     my $keep_conflict = $entry{'conflict'};
 
 #    print STDERR "Push_raw\n";
@@ -1266,6 +1283,7 @@ sub push_raw {
     $self->{po}{$msgid}{'msgstr'} = $msgstr;
     $self->{po}{$msgid}{'comment'} = $comment;
     $self->{po}{$msgid}{'automatic'} = $automatic;
+    $self->{po}{$msgid}{'previous'} = $previous;
     if (defined($self->{po}{$msgid}{'pos_doc'})) {
         $self->{po}{$msgid}{'pos_doc'} .= " ".$self->{count_doc}++;
     } else {
