@@ -36,7 +36,7 @@ areas where they were not expected like documentation.
 Locale::Po4a::Text is a module to help the translation of text documents into
 other [human] languages.
 
-Paragraphs are splitted on empty lines (or lines containing only spaces or
+Paragraphs are split on empty lines (or lines containing only spaces or
 tabulations).
 
 If a paragraph contains a line starting by a space (or tabulation), this
@@ -149,21 +149,21 @@ sub initialize {
     my $self = shift;
     my %options = @_;
 
-    my %valid = (
-        asciidoc => 1,
-        breaks => 1,
-        debianchangelog => 1,
-        debug => 1,
-        fortunes => 1,
-        markdown => 1,
-        nobullets => 1,
-        tabs => 1,
-        verbose => 1,
-    );
+    $self->{options}{'asciidoc'} = 1;
+    $self->{options}{'breaks'} = 1;
+    $self->{options}{'debianchangelog'} = 1;
+    $self->{options}{'debug'} = 1;
+    $self->{options}{'fortunes'} = 1;
+    $self->{options}{'markdown'} = 1;
+    $self->{options}{'nobullets'} = 1;
+    $self->{options}{'tabs'} = 1;
+    $self->{options}{'verbose'} = 1;
+
     foreach my $opt (keys %options) {
         die wrap_mod("po4a::text",
                      dgettext("po4a", "Unknown option: %s"), $opt)
-            unless exists $valid{$opt};
+            unless exists $self->{options}{$opt};
+        $self->{options}{$opt} = $options{$opt};
     }
 
     if (defined $options{'nobullets'}) {
@@ -202,7 +202,7 @@ sub parse {
     my $end_of_paragraph = 0;
     ($line,$ref)=$self->shiftline();
     my $file = $ref;
-    $file =~ s/:[0-9]+$//;
+    $file =~ s/:[0-9]+$// if defined($line);
     while (defined($line)) {
         $ref =~ m/^(.*):[0-9]+$/;
         if ($1 ne $file) {
