@@ -8,7 +8,7 @@
 # documents.
 #
 # Copyright (c) 2005 by Yves Rütschlé <po4a@rutschle.net>
-# Copyright (c) 2007 by Nicolas François <nicolas.francois@centraliens.net>
+# Copyright (c) 2007-2008 by Nicolas François <nicolas.francois@centraliens.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::Xhtml - Convert Xhtml documents from/to PO files
@@ -39,10 +41,6 @@ areas where they were not originally expected like documentation.
 Locale::Po4a::Xhtml is a module to help the translation of Xhtml documents into
 other [human] languages.
 
-Please note that this module is still experimental. It is not distributed in
-the official po4a releases since we don't feel it to be mature enough. If you
-insist on trying, check the CVS out.
-
 =head1 OPTIONS ACCEPTED BY THIS MODULE
 
 These are this module's particular options:
@@ -66,8 +64,6 @@ path to find files included by a B<virtual> attribute.
 This module is fully functional, as it relies in the L<Locale::Po4a::Xml>
 module. This only defines the translatable tags and attributes.
 
-It is derived from Jordi's DocBook module.
-
 "It works for me", which means I use it successfully on my personal Web site.
 However, YMMV: please let me know if something doesn't work for you. In
 particular, tables are getting no testing whatsoever, as we don't use them.
@@ -83,7 +79,8 @@ L<po4a(7)|po4a.7>, L<Locale::Po4a::TransTractor(3pm)>, L<Locale::Po4a::Xml(3pm)>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2004 by Yves Rütschlé <po4a@rutschle.net>
+ Copyright (c) 2004 by Yves Rütschlé <po4a@rutschle.net>
+ Copyright (c) 2007-2008 by Nicolas François <nicolas.francois@centraliens.net>
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
@@ -165,14 +162,24 @@ sub initialize {
         $self->{options}{'wrap'}=1;
         $self->{options}{'doctype'}=$self->{options}{'doctype'} || 'html';
 
-        $self->{options}{'inline'}.='
+        # Default tags are translated (text rewrapped), and introduce a
+        # break.
+        # The following list indicates the list of tags which should be
+        # translated without rewrapping.
+        $self->{options}{'_default_translated'}.='
+                W<pre>
+        ';
+
+        # The following list indicates the list of tags which should be
+        # translated inside the current block, whithout introducing a
+        # break.
+        $self->{options}{'_default_inline'}.='
                 <a> 
                 <abbr> 
                 <acronym> 
                 <b> 
                 <big> 
                 <bdo> 
-                <br> 
                 <button> 
                 <cite> 
                 <code> 
@@ -184,7 +191,6 @@ sub initialize {
                 <input> 
                 <kbd> 
                 <label> 
-                <map> 
                 <object> 
                 <q> 
                 <samp> 
@@ -196,6 +202,7 @@ sub initialize {
                 <sup> 
                 <textarea> 
                 <tt> 
+                <u>
                 <var> 
         ';
 
@@ -205,8 +212,10 @@ sub initialize {
         # write text (in which case you have bigger problems than this
         # program not inlining img: you now have to translate all your
         # images. That'll teach you).
+        # If you choose to translate images, you may also want to set
+        # <map> as placeholder and <area> as inline.
 
-        $self->{options}{'attributes'}.='
+        $self->{options}{'_default_attributes'}.='
                 alt
                 lang
                 title
